<?php

class DemoController extends CController {
	public function beforeAction($action) {
		$config = array();
		switch ($action->id) {
			case 'registration':
				$config = array(
					'steps'=>array('user', 'userProfile', 'contactDetails', 'paymentDetails'),
					'events'=>array(
						'onStart'=>'wizardStart',
						'onProcessStep'=>'registrationWizardProcessStep',
						'onFinished'=>'wizardFinished',
						'onInvalidStep'=>'wizardInvalidStep',
						'onSaveDraft'=>'wizardSaveDraft'
					),
					'menuLastItem'=>'Register'
				);
				break;
			case 'quiz':
				$config = array(
					'steps'=>array('Q1'=>'question_1','Q2'=>'question_2','Q3'=>'question_3','Q4'=>'question_4','Q5'=>'question_5','Q6'=>'question_6','Q7'=>'question_7','Q8'=>'question_8','Q9'=>'question_9','Q10'=>'question_10'),
					'timeout'=>30,
					'continueOnExpired'=>true,
					'forwardOnly'=>true,
					'events'=>array(
						'onStart'=>'wizardStart',
						'onProcessStep'=>'quizProcessStep',
						'onFinished'=>'quizFinished',
						'onInvalidStep'=>'wizardInvalidStep',
						'onExpiredStep'=>'quizExpiredStep'
					)
				);
				break;
			case 'survey':
				$config = array(
					'steps'=>array('pet',array('hasPet'=>array('rescued', array('isRescued'=>'isRescued', 'notRescued'=>'notRescued'))),array('dog'=>array('dogGroup','dogPlay'),'cat'=>array('catGroup','catPlay'),'dog and cat'=>array('dogGroup','dogPlay','catGroup','catPlay'),'neither'=>'noPet'),array('hasPet'=>array('neutered', array('isNeutered'=>'isNeutered', 'notNeutered'=>'notNeutered')))),
					'defaultBranch'=>false,
					'events'=>array(
						'onStart'=>'wizardStart',
						'onProcessStep'=>'surveyWizardProcessStep',
						'onFinished'=>'wizardFinished',
						'onInvalidStep'=>'wizardInvalidStep'
					),
					'menuLastItem'=>'Completed'
				);
				break;
			case 'resumeRegistration':
				$config['steps'] = array(); // only here to force attachment of WizardBehavior
			default:
				break;
		}
		if (!empty($config)) {
			$config['class']='application.components.WizardBehavior';
			$this->attachBehavior('wizard', $config);
		}
		return parent::beforeAction($action);
	}

	public function actionIndex() {
		$this->render('index');
	}

	public function actionRegistration($step=null) {
		$this->pageTitle = 'Registration Wizard';
		$this->process($step);
	}

	public function actionSurvey($step=null) {
		$this->pageTitle = 'Survey Wizard';
		$this->process($step);
	}

	public function actionQuiz($step=null) {
		$this->pageTitle = 'Quiz Wizard';
		$this->process($step);
	}

	/**
	 * Resumes a registration.
	 */
	public function actionResumeRegistration($uuid) {
		$user = new User();
		$data = $user->recoverRegistration($uuid);
		if ($data===false || !$this->wizard->restore($data))
			Yii::app()->getUser()->setFlash('notice', 'Your registration data could not be recovered.');
		else
			Yii::app()->getUser()->setFlash('success', 'Your registration has been recovered; please continue.');
		$this->redirect(array('registration'));
	}

	// Wizard Behavior Event Handlers
	/**
	* Raised when the wizard starts; before any steps are processed.
	* MUST set $event->handled=true for the wizard to continue.
	* Leaving $event->handled===false causes the onFinished event to be raised.
	* @param WizardEvent The event
	*/
	public function wizardStart($event) {
		$event->handled = true;
	}

	/**
	* Raised when the wizard detects an invalid step
	* @param WizardEvent The event
	*/
	public function wizardInvalidStep($event) {
		Yii::app()->getUser()->setFlash('notice', $event->step.' is not a vaild step in this wizard');
	}

	/**
	* The wizard has finished; use $event->step to find out why.
	* Normally on successful completion ($event->step===true) data would be saved
	* to permanent storage; the demo just displays it
	* @param WizardEvent The event
	*/
	public function wizardFinished($event) {
		if ($event->step===true)
			$this->render('completed', compact('event'));
		else
			$this->render('finished', compact('event'));

		$event->sender->reset();
		Yii::app()->end();
	}

	/**
	* Saves a draft of the wizard that can be resumed at a later time
	* @param WizardEvent $event
	*/
	public function wizardSaveDraft($event) {
		$user = new User();
		$uuid = $user->saveRegistration($event->data);
		$event->sender->reset();
		$this->render('draft',compact('uuid'));
		Yii::app()->end();
	}

	/**
	* Process wizard steps.
	* The event handler must set $event->handled=true for the wizard to continue
	* @param WizardEvent The event
	*/
	public function registrationWizardProcessStep($event) {
		$modelName = ucfirst($event->step);
		$model = new $modelName();
		$model->attributes = $event->data;
		$form = $model->getForm();

		// Note that we also allow sumission via the Save button
		if (($form->submitted()||$form->submitted('save_draft')) && $form->validate()) {
			$event->sender->save($model->attributes);
			$event->handled = true;
		}
		else
			$this->render('form', compact('event','form'));
	}

	/**
	* Process wizard steps.
	* The event handler must set $event->handled=true for the wizard to continue
	* @param WizardEvent The event
	*/
	public function surveyWizardProcessStep($event) {
		$modelName = ucfirst($event->step);
		$model = new $modelName();
		$model->attributes = $event->data;
		$form = $model->getForm();

		if (($form->submitted()||$form->submitted('save_draft')) && $form->validate()) {
			$event->sender->save(empty($model->attributes)?array():$model->attributes);
			$event->handled = true;

			switch ($event->step) {
				case 'pet':
					$branches = array(
		        'dog'=>WizardBehavior::BRANCH_DESELECT,
		        'cat'=>WizardBehavior::BRANCH_DESELECT,
		        'dog and cat'=>WizardBehavior::BRANCH_DESELECT,
		        'neither'=>WizardBehavior::BRANCH_DESELECT,
		        'hasPet'=>WizardBehavior::BRANCH_SKIP
		      );
					$branches[strtolower($model->type)] = WizardBehavior::BRANCH_SELECT;
					if ($model->type!=='Neither')
						$branches['hasPet'] = WizardBehavior::BRANCH_SELECT;
					$event->sender->branch($branches);
					break;
				case 'rescued':
					$branches = array(
		        'isRescued'=>WizardBehavior::BRANCH_DESELECT,
		        'notRescued'=>WizardBehavior::BRANCH_DESELECT
		      );
		      if ($model->answer==='Yes')
		      	$branches['isRescued'] = WizardBehavior::BRANCH_SELECT;
		      else
		       	$branches['notRescued'] = WizardBehavior::BRANCH_SELECT;
					break;
				case 'neutered':
					$branches = array(
		        'isNeutered'=>WizardBehavior::BRANCH_DESELECT,
		        'notNeutered'=>WizardBehavior::BRANCH_DESELECT
		      );
		      if ($model->answer==='Yes')
		      	$branches['isNeutered'] = WizardBehavior::BRANCH_SELECT;
		      else
		       	$branches['notNeutered'] = WizardBehavior::BRANCH_SELECT;
					break;
				default:
					break;
			}
			if (!empty($branches))
				$event->sender->branch($branches);
		}
		else
			$this->render('form', compact('event','form'));
	}

	/**
	* Process steps from the quiz
	* @param WizardEvent The event
	*/
	public function quizProcessStep($event) {
		$modelName = ucfirst(str_replace('_','',$event->step));
		$model = new $modelName();
		$form = $model->getForm();
		if ($form->submitted() && $form->validate()) {
			$event->sender->save(array('answer'=>$model->answer));
			$event->handled = true;
		}
		else
			$this->render('quiz/form', compact('event','form'));
	}

	/**
	* The quiz
	* @param WizardEvent The event
	*/
	public function quizFinished($event) {
		$this->render('quiz/completed', compact('event'));
		$event->sender->reset();
		Yii::app()->end();
	}

	/**
	* Raised when a step has expired.
	* @param WizardEvent The event
	*/
	public function quizExpiredStep($event) {
		$event->sender->save(array('answer'=>'<slow>'));
	}
}